//pngStitchFrame.cpp - png2ico frame class

#include "wx/wxprec.h"

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "wx/image.h"

#include "pngStitchFrame.h"

#ifdef __WXMSW__
#ifdef _DEBUG
#define new DEBUG_NEW
#endif
#endif

BEGIN_EVENT_TABLE(pngStitchFrame, wxFrame)
	EVT_SIZE(pngStitchFrame::OnSize)
	EVT_BUTTON(LOAD, pngStitchFrame::OnLoad)
	EVT_BUTTON(UP, pngStitchFrame::OnUp)
	EVT_BUTTON(DOWN, pngStitchFrame::OnDown)
	EVT_BUTTON(DELETEFILES, pngStitchFrame::OnDelete)
	EVT_BUTTON(REVERSE, pngStitchFrame::OnReverse)
	EVT_BUTTON(GETOPFILEPATH, pngStitchFrame::OnSave)
	EVT_BUTTON(DOIT, pngStitchFrame::OnDoIt)
END_EVENT_TABLE()


pngStitchFrame::pngStitchFrame(const wxChar *title,
							   int xpos,
							   int ypos,
							   int width,
							   int height):
wxFrame((wxFrame *)NULL,
		-1,
		title,
		wxPoint(xpos, ypos),
		wxSize(width, height),
		(wxMINIMIZE_BOX|
		 wxMAXIMIZE_BOX|
		 wxCLOSE_BOX|
		 wxSYSTEM_MENU|
		 wxCAPTION|
		 wxRESIZE_BORDER))
{
	SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNFACE));

	wxBoxSizer *ButtonSizer = new wxBoxSizer(wxVERTICAL);

	Load = new wxButton(this, LOAD, wxT("Load"), wxPoint(10, 10));
	Up = new wxButton(this, UP, wxT("Up"), wxPoint(10, 40));
	Down = new wxButton(this, DOWN, wxT("Down"), wxPoint(10, 70));
	Delete = new wxButton(this, DELETEFILES, wxT("Delete"), wxPoint(10, 100));
	Reverse = new wxButton(this, REVERSE, wxT("Reverse"), wxPoint(10, 130));

	ButtonSizer->Add(Load, 0, wxALL, 4);
	ButtonSizer->Add(Up, 0, wxALL, 4);
	ButtonSizer->Add(Down, 0, wxALL, 4);
	ButtonSizer->Add(Delete, 0, wxALL, 4);
	ButtonSizer->Add(Reverse, 0, wxALL, 4);

	Images = new wxListBox(this,
						   IMAGES,
						   wxPoint(100, 10),
						   wxSize(170, 210),
						   0,
						   NULL,
						   (wxLB_MULTIPLE|wxLB_NEEDED_SB|wxLB_HSCROLL));

	wxStaticText *FPLabel = new wxStaticText(this, -1, wxT("Output File Path :"), wxPoint(280, 10));

	OPFilePath = new wxTextCtrl(this, -1, wxT(""), wxPoint(280, 30), wxSize(80, 22));
	GetOPFilePath = new wxButton(this, GETOPFILEPATH, wxT("..."), wxPoint(370, 30), wxSize(20, 22));

	DoIt = new wxButton(this, DOIT, wxT("Do It"), wxPoint(280, 60));

	wxBoxSizer *FPSizer = new wxBoxSizer(wxHORIZONTAL);
	FPSizer->Add(OPFilePath, 0, wxALL, 4);
	FPSizer->Add(GetOPFilePath, 0, wxALL, 4);

	wxBoxSizer *RightSizer = new wxBoxSizer(wxVERTICAL);
	RightSizer->Add(FPLabel, 0, wxALL, 4);
	RightSizer->Add(FPSizer, 0, wxALL, 2);
	RightSizer->Add(DoIt, 0, wxALL, 4);

	wxBoxSizer *DialogSizer = new wxBoxSizer(wxHORIZONTAL);

	DialogSizer->Add(ButtonSizer, 0, wxALL, 4);
	DialogSizer->Add(Images, 1, wxALL|wxEXPAND, 4);
	DialogSizer->Add(RightSizer, 0, wxALL, 4);
	SetSizer(DialogSizer);
	DialogSizer->SetSizeHints(this);
	//SetAutoLayout(TRUE);
	//Layout();

	//draw an arrow from the arrow png
	wxInitAllImageHandlers();

	LoadDlg = 0;
	LoadDlg = new wxFileDialog(this,
							   wxT("Open a .png file..."),
							   wxGetCwd(),
							   wxT(""),
							   wxT("*.png"),
							   (wxOPEN|wxMULTIPLE));
	SaveDlg = 0;
	SaveDlg = new wxFileDialog(this,
							   wxT("Write to a .png file..."),
							   wxGetCwd(),
							   wxT(""),
							   wxT("*.png"),
							   wxSAVE);
//------------------------------------------------------------------------------
#ifdef WIN32
	wxIcon icon(wxICON(PNGSTITCH));
	SetIcon(icon);
#endif
}

pngStitchFrame::~pngStitchFrame()
{
	if(LoadDlg)
		delete LoadDlg;
	if(SaveDlg)
		delete SaveDlg;
}

void pngStitchFrame::OnSize(wxSizeEvent& event)
{
	Layout();
}

void pngStitchFrame::OnLoad(wxCommandEvent &event)
{
	int i, j;
	bool alreadygotit = false;

	wxArrayString tempstrings;

	if(LoadDlg->ShowModal() == wxID_OK)
	{
		//FilePath1->SetValue(LoadDlg->GetPath());
		LoadDlg->GetPaths(tempstrings);

		for(i=0;i<tempstrings.GetCount();i++)
		{
			alreadygotit = false;
			for(j=0;j<InputFiles.GetCount();j++)
			{
				if(InputFiles.Item(j) == tempstrings.Item(i))
				{
					alreadygotit = true;
				}
			}
			if(!alreadygotit)
			{
				InputFiles.Add(tempstrings.Item(i));
				Images->InsertItems(1, &(InputFiles.Item(InputFiles.GetCount()-1)), Images->GetCount());
			}
		}
	}
}

void pngStitchFrame::OnUp(wxCommandEvent &event)
{
	wxArrayInt tempselection;
	wxString tempstring;

	Images->GetSelections(tempselection);
	if(tempselection.GetCount() > 0)
	{
		if(tempselection.Item(0) > 0)
		{
			tempstring = InputFiles.Item(tempselection.Item(0));
			InputFiles.Item(tempselection.Item(0)) = InputFiles.Item(tempselection.Item(0)-1);
			InputFiles.Item(tempselection.Item(0)-1) = tempstring;

			Images->Set(InputFiles);
			Images->SetSelection(tempselection[0]-1);
		}
	}
}

void pngStitchFrame::OnDown(wxCommandEvent &event)
{
	wxArrayInt tempselection;
	wxString tempstring;

	Images->GetSelections(tempselection);
	if(tempselection.GetCount() > 0)
	{
		if(tempselection.Item(0) < (InputFiles.GetCount()-1))
		{
			tempstring = InputFiles.Item(tempselection.Item(0));
			InputFiles.Item(tempselection.Item(0)) = InputFiles.Item(tempselection.Item(0)+1);
			InputFiles.Item(tempselection.Item(0)+1) = tempstring;

			Images->Set(InputFiles);
			Images->SetSelection(tempselection[0]+1);
		}
	}
}

void pngStitchFrame::OnDelete(wxCommandEvent &event)
{
	int i;
	wxArrayInt tempselection;

	Images->GetSelections(tempselection);
	if(tempselection.GetCount() > 0)
	{
		for(i=(tempselection.GetCount()-1);i>=0;i--)
		{
			InputFiles.Remove(tempselection.Item(i));
		}
		Images->Set(InputFiles);
	}
}

void pngStitchFrame::OnReverse(wxCommandEvent &event)
{
	long i;
	wxArrayString temparr;

	for(i=(InputFiles.GetCount()-1);i>=0;i--)
		temparr.Add(InputFiles[i]);

	InputFiles.Clear();
	for(i=0;i<temparr.GetCount();i++)
		InputFiles.Add(temparr[i]);

	Images->Clear();
	for(i=0;i<InputFiles.GetCount();i++)
		Images->Append(InputFiles[i]);
}

void pngStitchFrame::OnSave(wxCommandEvent &event)
{
	if(SaveDlg->ShowModal() == wxID_OK)
	{
		OPFilePath->SetValue(SaveDlg->GetPath());
	}
}

void pngStitchFrame::OnDoIt(wxCommandEvent &event)
{
	FILE *fp;
	char buf[PNG_BYTES_TO_CHECK];
	char tempstr[256];
	wxString tempwxstr;
	int i, j;
    png_uint_32 width, height, finalheight;
	int bit_depth, color_type, interlace_type;
	png_structp png_ptr;
    png_infop info_ptr, final_info_ptr;
	int is_png;
	png_bytep *rowpointers;
	long rpindex = 0;
	png_bytep *rptemp;

	finalheight = 0;

	if(Images->GetCount() == 0)
		return;
	if(OPFilePath->GetValue() == wxT(""))
		return;

	for(i=0;i<Images->GetCount();i++) //this is where we work out the height of all the images put together
	{
		tempwxstr = Images->GetString(i); //this should be the input files, not the output file.
		//strcpy(tempstr, tempwxstr.GetData());
		strcpy(tempstr, tempwxstr.mb_str());

		if((fp = fopen(tempstr, "rb")) == NULL)
		  continue;

		fread(buf, 1, PNG_BYTES_TO_CHECK, fp);

		is_png = png_sig_cmp((unsigned char *)buf, (png_size_t)0, PNG_BYTES_TO_CHECK);
		if(is_png)
		{
			fclose(fp);
			continue;
		}

		png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
		if (png_ptr == NULL)
		{
			fclose(fp);
			continue;
		}

		//Allocate/initialize the memory for image information.  REQUIRED.
		info_ptr = png_create_info_struct(png_ptr);
		if (info_ptr == NULL)
		{
			fclose(fp);
			png_destroy_read_struct(&png_ptr, png_infopp_NULL, png_infopp_NULL);
			continue;
		}

#ifndef PNG_SETJMP_NOT_SUPPORTED
		if(setjmp(png_jmpbuf(png_ptr)))
		{
			png_destroy_read_struct(&png_ptr, &info_ptr, png_infopp_NULL);
			fclose(fp);

			continue; //If we get here, we had a problem reading the file
		}
#endif
		png_init_io(png_ptr, fp);
		png_set_sig_bytes(png_ptr, 4); //because if we get here, we're ok

		if(i == 0)
		{
			final_info_ptr = info_ptr;

			png_read_info(png_ptr, info_ptr);

			png_get_IHDR(png_ptr, info_ptr, &width, &height, &bit_depth, &color_type, &interlace_type, int_p_NULL, int_p_NULL);

			png_bytep *rowpointers2 = new png_bytep[height];
			for(long row=0;row<height;row++)
			{
				rowpointers2[row] = (unsigned char *)png_malloc(png_ptr, png_get_rowbytes(png_ptr, info_ptr));
			}
			png_read_image(png_ptr, rowpointers2);

			delete [] rowpointers2;
		}
		else
		{
			//this is where we read the image
			png_read_png(png_ptr, info_ptr, PNG_TRANSFORM_IDENTITY, png_voidp_NULL);
		}

		//add the current image's height on to the rest.
		finalheight += png_get_image_height(png_ptr, info_ptr);
		if(i==0)
			width = png_get_image_width(png_ptr, info_ptr);

		png_destroy_read_struct(&png_ptr, &info_ptr, png_infopp_NULL);

		fclose(fp);
	}

	rowpointers = new png_bytep[finalheight];

	for(i=0;i<Images->GetCount();i++) //this is where we put all the image data in the one array
	{
		tempwxstr = Images->GetString(i); //this should be the input files, not the output file.
		//strcpy(tempstr, tempwxstr.GetData());
		strcpy(tempstr, tempwxstr.mb_str());

		if((fp = fopen(tempstr, "rb")) == NULL)
		  continue;

		fread(buf, 1, PNG_BYTES_TO_CHECK, fp);

		is_png = png_sig_cmp((unsigned char *)buf, (png_size_t)0, PNG_BYTES_TO_CHECK);
		if(is_png)
		{
			fclose(fp);
			continue;
		}

		png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
		if (png_ptr == NULL)
		{
			fclose(fp);
			continue;
		}

		//Allocate/initialize the memory for image information.  REQUIRED.
		info_ptr = png_create_info_struct(png_ptr);
		if (info_ptr == NULL)
		{
			fclose(fp);
			png_destroy_read_struct(&png_ptr, png_infopp_NULL, png_infopp_NULL);
			continue;
		}

#ifndef PNG_SETJMP_NOT_SUPPORTED
		if(setjmp(png_jmpbuf(png_ptr)))
		{
			png_destroy_read_struct(&png_ptr, &info_ptr, png_infopp_NULL);
			fclose(fp);

			continue; //If we get here, we had a problem reading the file
		}
#endif
		png_init_io(png_ptr, fp);
		png_set_sig_bytes(png_ptr, 4); //because if we get here, we're ok

		//this is where we read the image
		//png_read_png(png_ptr, info_ptr, PNG_TRANSFORM_IDENTITY, png_voidp_NULL);

		png_read_info(png_ptr, info_ptr);

		for(j=0;j<png_get_image_height(png_ptr, info_ptr);j++)
		{
			rowpointers[j+rpindex] = (unsigned char *)png_malloc(png_ptr, png_get_rowbytes(png_ptr,info_ptr));
		}

		//rptemp = &(rowpointers[rpindex]);
		//(rowpointers + rpindex) = png_get_rows(png_ptr, info_ptr);
		for(j=0;j<png_get_image_height(png_ptr, info_ptr);j++)
		{
			png_read_rows(png_ptr, &rowpointers[j+rpindex], png_bytepp_NULL, 1);
		}
		png_read_end(png_ptr, info_ptr);

		rpindex += png_get_image_height(png_ptr, info_ptr);

		png_destroy_read_struct(&png_ptr, &info_ptr, png_infopp_NULL);

		fclose(fp);
	}

	//...and now we write the output file
	tempwxstr = OPFilePath->GetValue(); //this should be the input files, not the output file.
	//strcpy(tempstr, tempwxstr.GetData());
	strcpy(tempstr, tempwxstr.mb_str());
	fp = fopen(tempstr, "wb");
	if(!fp)
		return;

	png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
	if(!png_ptr)
	{
		fclose(fp);
		return;
	}

	final_info_ptr = png_create_info_struct(png_ptr);
	if (final_info_ptr == NULL)
	{
		fclose(fp);
		png_destroy_write_struct(&png_ptr,  png_infopp_NULL);
		return;
	}

#ifndef PNG_SETJMP_NOT_SUPPORTED
	if(setjmp(png_jmpbuf(png_ptr)))
	{
		png_destroy_write_struct(&png_ptr, &final_info_ptr);
		fclose(fp);
		return;
	}
#endif

	png_init_io(png_ptr, fp);

	png_set_IHDR(png_ptr, final_info_ptr, width, finalheight, bit_depth, color_type, interlace_type, PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);

	png_write_info(png_ptr, final_info_ptr);

	png_write_image(png_ptr, rowpointers);

	png_write_png(png_ptr, final_info_ptr, PNG_TRANSFORM_IDENTITY, png_voidp_NULL);

	png_write_end(png_ptr, final_info_ptr);

	fclose(fp);

	delete [] rowpointers;
}
